@@ -1,7 +1,8 @@
-# ClamTk, copyright (C) 2004-2021 Dave M
+# ClamTk, copyright (C) 2004-2024 Dave M
 #
 # This file is part of ClamTk
-# (https://gitlab.com/dave_m/clamtk-gtk3/).
+# https://github.com/dave-theunsub/clamtk/
+# https://gitlab.com/dave_m/clamtk/
 #
 # ClamTk is free software; you can redistribute it and/or modify it
 # under the terms of either:
@@ -36,10 +37,10 @@ my $found;    # Holds information of bad stuff found
 
 my $found_count = 0;    # Scalar number of bad stuff found
 my $num_scanned = 0;    # Overall number of files scanned
-my %dirs_scanned;       # Directories scanned
+my $dirs_scanned;       # Directories scanned
 
-my $hb;                 # HeaderBar
-my $pb;                 # ProgressBar
+my $hb;                     # HeaderBar
+my $pb;                     # ProgressBar
 my $pb_file_counter = 0;    # For ProgressBar
 my $pb_step;                # For ProgressBar
 my $root_scan = FALSE;      # Scanning /
@@ -56,6 +57,7 @@ my $window;                 # Main window/dialog
 my $from_cli;               # from the commandline?
 
 sub filter {
+
     # $pkg_name = drop this
     # $scanthis = file or directory to be scanned
     # $show = whether or not to show the preferences button;
@@ -88,12 +90,10 @@ sub filter {
     }
 
     # Begin popup scanning
-    $window = Gtk3::Dialog->new(
-        undef, undef,
-        [   qw| modal destroy-with-parent no-separator
-                use-header-bar |
-        ],
-    );
+    $window
+        = Gtk3::Dialog->new( undef, undef,
+        [ qw| modal destroy-with-parent use-header-bar | ],
+        );
     $window->set_deletable( FALSE );
     $window->set_default_size( 450, 80 );
 
@@ -152,8 +152,7 @@ sub filter {
         = Gtk3::Label->new( sprintf _( "Files scanned: %d" ), $num_scanned );
     $files_scanned_label->set_alignment( 0.0, 0.5 );
 
-    $threats_label
-        = Gtk3::Label->new( sprintf _( "Possible threats: %d" ),
+    $threats_label = Gtk3::Label->new( sprintf _( "Possible threats: %d" ),
         $found_count );
     $threats_label->set_alignment( 0.0, 0.5 );
 
@@ -165,13 +164,15 @@ sub filter {
     $box->pack_start( $bottombar, FALSE, FALSE, 5 );
 
     $bottombar->set_message_type( 'other' );
+
     # Stupid infobars
     # $use_image = ClamTk::Icons->get_image( 'gtk-cancel' );
     $use_image = 'gtk-cancel';
     $bottombar->add_button( $use_image, HATE_GNOME_SHELL );
     if ( $show ) {
+
         # $use_image = ClamTk::Icons->get_image( 'preferences-system' );
-        $use_image = ClamTk::Icons->get_image( 'document-properties' );
+        $use_image = ClamTk::Icons->get_image( 'gtk-preferences' );
         $bottombar->add_button( $use_image, DESTROY_GNOME_SHELL );
     }
     $bottombar->signal_connect(
@@ -248,12 +249,14 @@ sub filter {
         Mail	kmail   evolution   timeshift
     );
     for my $mailbox ( @maildirs ) {
+
         # warn "excluding mailbox directory $mailbox\n";
         $directive .= " --exclude-dir=$mailbox";
     }
 
     # remove the hidden files if chosen:
     if ( !$prefs{ ScanHidden } ) {
+
         # But only if Trash directory is not being scanned
         if ( $scanthis !~ m#/.local/share/Trash# ) {
             $directive .= ' --exclude="\/\."';
@@ -262,14 +265,16 @@ sub filter {
 
     # symlinks:
     # The symlink stuff from clamscan requires >= 0.97.
+    # Symlinks should probably be a preference though;
+    # just assuming that most people want them followed.
     my ( $version ) = ClamTk::App->get_AV_version();
+
     # Ensure it's just digits and dots:
     $version =~ s/[^0-9\.]//g;
     $version =~ s/^[0\.]+//;
-    $version =~ s/\.0$//;
-    if (   ( $version <=> '97' ) == 0
-        || ( $version <=> '97' ) == 1 )
-    {
+    $version =~ s/\.//g;
+
+    if ( $version >= '97' ) {
         $directive .= ' --follow-dir-symlinks=1';
         $directive .= ' --follow-file-symlinks=1';
     }
@@ -285,22 +290,9 @@ sub filter {
     # potentially unwanted applications and broken executables.
     # Version 0.101.0 changed and added some options.
     if ( $prefs{ Thorough } ) {
-        if (   ( $version <=> '101' ) == 0
-            || ( $version <=> '101' ) == 1 )
-        {
-            $directive .= ' --detect-pua --alert-broken'
-            . ' --alert-macros';
-        } else {
-            $directive .= ' --detect-pua --algorithmic-detection';
-        }
+        $directive .= ' --detect-pua --alert-broken' . ' --alert-macros';
     } else {
-        if (   ( $version cmp '101' ) == 0
-            || ( $version cmp '101' ) == 1 )
-        {
-            $directive =~ s/\s--detect-pua --alert-broken --alert-macros'//;
-        } else {
-            $directive =~ s/\s--detect-pua --algorithmic-detection//;
-        }
+        $directive =~ s/\s--detect-pua --alert-broken --alert-macros'//;
     }
 
     # Heuristic scanning
@@ -375,6 +367,10 @@ sub scan {
     #>>>
     # binmode( $SCAN, ':utf8:bytes' );
 
+    # This is a placeholder for use in keeping track of
+    # directories scanned
+    my $dirname;
+
     Gtk3::main_iteration while Gtk3::events_pending;
     while ( <$SCAN> ) {
         chomp;
@@ -387,11 +383,16 @@ sub scan {
 
         if ( /^Scanning (.*?)$/ ) {
             my $base = decode( 'UTF-8', basename( $1 ) );
+            $dirname = decode( 'UTF-8', dirname( $1) );
+
+            # Lots of temporary things under /tmp/clamav;
+            # we'll just ignore them.
+            $dirs_scanned->{ $dirname } = 1;
+
             # Display stuff in popup infobar
             set_infobar_text(
                 $topbar,
                 # sprintf( _( 'Scanning %s...' ), $dirname )
-                # sprintf( _( 'Scanning %s...' ), $dirname )
                 sprintf( _( 'Scanning %s...' ), $base )
             );
             $num_scanned++;
@@ -431,7 +432,6 @@ sub scan {
         chomp( $file )   if ( defined $file );
         chomp( $status ) if ( defined $status );
 
-        my $dirname   = decode( 'UTF-8', dirname( $file ) );
         my $fileparse = decode( 'UTF-8', fileparse( $file ) );
 
         my $hidden = ClamTk::Prefs->get_preference( 'ScanHidden' );
@@ -444,12 +444,6 @@ sub scan {
             $dirparse = $dirname;
         }
 
-        # Lots of temporary things under /tmp/clamav;
-        # we'll just ignore them.
-        $dirs_scanned{ $dirname } = 1
-            unless ( dirname( $file ) =~ /\/tmp\/clamav/
-            || dirname( $file ) eq '.' );
-
         # Do not show files in archives - we just want the end-result.
         # It still scans and we still show the result.
         next if ( $file =~ /\/tmp\/clamav/ );
@@ -480,7 +474,7 @@ sub scan {
     }
 
     Gtk3::main_iteration while Gtk3::events_pending;
-
+    
     # Done scanning - close filehandle and return to
     # filter() and then to clean-up
     close( $SCAN );    # or warn "Unable to close scanner! $!\n";
@@ -537,12 +531,14 @@ sub destroy_progress {
 }
 
 sub reset_stats {
+
     # reset things
     $num_scanned     = 0;
     $found_count     = 0;
     $pb_file_counter = 0;
     $pb_step         = 0;
-    %dirs_scanned    = ();
+    $dirs_scanned    = ();
+    $found           = ();
     $stopped         = 1;
     $directive       = '';
     $root_scan       = FALSE;
@@ -551,7 +547,7 @@ sub reset_stats {
 sub bad_popup {
     my $dialog = Gtk3::MessageDialog->new(
         $window, [ qw| modal destroy-with-parent | ],
-        'info', 'close', _( 'No threats found' ),
+        'info',  'close', _( 'No threats found' ),
     );
     $dialog->run;
     $dialog->destroy;
@@ -559,6 +555,7 @@ sub bad_popup {
 
 sub logit {
     my $db_total = ClamTk::App->get_sigtool_info( 'count' );
+
     # warn "in Scan: db_total = >$db_total<\n";
     my $REPORT;    # filehandle for histories log
 
@@ -578,11 +575,11 @@ sub logit {
     my $paths = ClamTk::App->get_path( 'history' );
 
     my $virus_log
-        = $paths . "/" . decode( 'utf8', "$mon-$day-$year" ) . '.log';
+        = $paths . "/" . decode( 'UTF-8', "$mon-$day-$year" ) . '.log';
 
     #<<<
     # sort the directories scanned for display
-    my @sorted = sort { $a cmp $b } keys %dirs_scanned;
+    my @sorted = sort { $a cmp $b } keys %$dirs_scanned;
     if ( open $REPORT, '>>:encoding(UTF-8)', $virus_log ) {
         print $REPORT "\nClamTk, v",
             ClamTk::App->get_TK_version(), "\n",
@@ -625,9 +622,11 @@ sub logit {
                 ? length( $found->{ $length }->{ status } )
                 : $rsize;
         }
+
         # Set a buffer which is probably unnecessary.
         $lsize += 5;
         $rsize += 5;
+
         # Print to the log:
         for my $num ( sort keys %$found ) {
             printf $REPORT "%-${lsize}s %-${rsize}s\n",
@@ -669,6 +668,7 @@ sub set_infobar_text {
 }
 
 sub add_default_buttons {
+
     # We're going to show the following:
     # cancel-button: obviously cancels the scan
     # prefs-button: allows popup 'clamtk' with no args, for settings.
